/*
 * VF_Comp.c
 *
 *  Programmmed by Hirotsugu Kakugawa, Hiroshima University
 *  E-Mail:  kakugawa@se.hiroshima-u.ac.jp
 *
 *  Edition History
 *   5 Nov 1993  
 *  20 Jan 1994  Added GetOutline2().
 *   8 Mar 1994  Fixed small bugs and removed dead code
 */


/* This file is part of VFlib
 *
 * Copyright (C) 1993 Hirotsugu KAKUGAWA.   All rights reserved.
 *
 * VFlib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
 * to anyone for the consequences of using it or for whether it serves any
 * particular purpose or works at all, unless he says so in writing.  Refer
 * to the GNU General Public License for full details.
 *
 * Everyone is granted permission to copy, modify and redistribute
 * VFlib, but only under the conditions described in the GNU
 * General Public License.  A copy of this license is supposed to have been
 * given to you along with VFlib so you can know your rights and
 * responsibilities.  It should be in a file named COPYING.  Among other
 * things, the copyright notice and this notice must be preserved on all
 * copies. 
 */


/*
  Capabilities interpretable by compound font objects::
  "kn"  (str)  --  Kana Font Entry
  "kj"  (str)  --  Kanji Font Entry
  "sy"  (str)  --  Symbol Font Entry
 */

#include  <stdio.h> 
#include  <stdlib.h> 
#include  "config.h"
#include  "_VF.h"
#include  "VF.h"
#include  "VFcap.h"

#ifndef MAX
#define MAX(a, b) ( ((a)>(b)) ? (a) : (b) )
#endif
#ifndef MIN
#define MIN(a, b) ( ((a)<(b)) ? (a) : (b) )
#endif

#define RANGE_SET_MAX 32
#define P_FONT_MAX 16

struct s_range {
    int start;
    int end;
};
typedef struct s_range  Range;
typedef struct s_range* RangeSet;
static Range     NULL_Range       = { -1, -1 };
static RangeSet NULL_RangeSet   = { &NULL_Range };

struct s_p_font {
  char      *name;
  int       FD;
  RangeSet range_set;
};
typedef struct s_p_font P_FONT;

struct s_font {
  int     fd;
  P_FONT  p_font_set[P_FONT_MAX];
};
typedef struct s_font  Font;


Private int    OpenFont();
Private int    CloseFont();
Private int    GetBitmap();
Private long*  GetOutline();
Private long*  GetOutline2();
Private int    DrawOutline();
Private int    FreeOutline();
Private int    Link();
Private int    Unlink();

Private  int ReadCapa();
Private  RangeSet parse_range_set();
Private  int is_in();



Public FontObj*
CreateFont_Comp2(ent)
  char *ent;
{
  Font    *font;
  FontObj *fobj;

  if ((font = (Font*) malloc(sizeof(Font))) == NULL)
    return NULL;  /* ERR: malloc err */
  font->fd = -1;
  if (ReadCapa(font, ent) < 0)
    return NULL;

  fobj = (FontObj*) malloc(sizeof(FontObj));
  fobj->ClassID     = VF_FONT_COMPOUND2;
  fobj->Self        = fobj;
  fobj->LinkCount   = 0;
  fobj->OpenFont    = OpenFont;
  fobj->CloseFont   = CloseFont;
  fobj->GetBitmap   = GetBitmap;
  fobj->GetOutline  = GetOutline;
  fobj->GetOutline2 = GetOutline2;
  fobj->DrawOutline = DrawOutline;
  fobj->FreeOutline = FreeOutline;
  fobj->GetCharSet  = NULL;
  fobj->GetEnc      = NULL;
  fobj->Link        = Link;
  fobj->Unlink      = Unlink;
  fobj->Locals      = (long) font;
  return fobj;
}
       

Private int
OpenFont(obj)
  FontObj* obj;
{
  Font  *font;
  int i;

  font = (Font*) obj->Locals;
  for (i = 0; i < P_FONT_MAX; i++) {
      if (font->p_font_set[i].name != NULL)
	  font->p_font_set[i].FD = VF_OpenFont(font->p_font_set[i].name);
  }
  return 0;
}


Private int
CloseFont(obj, fid)
  FontObj  *obj;
  int      fid;
{
  Font  *font;
  int i;

  font = (Font*) obj->Locals;
  for (i = 0; i < P_FONT_MAX; i++) {
      if (font->p_font_set[i].name != NULL)
	  font->p_font_set[i].FD = VF_CloseFont(font->p_font_set[i].FD);
  }
  return 0;
}

Private int
GetBitmap(obj, jiscode, w, h, bw, bo, bm_buf)
  FontObj  *obj;
  int   jiscode;
  int   w;
  int   h;
  int   bw;
  int   bo;
  char  *bm_buf;
{
  int   val = -1;
  Font  *font;
  int i;
  
  font = (Font*) obj->Locals;
  for (i = 0; i < P_FONT_MAX; i++) {
      if (is_in(font->p_font_set[i].range_set, jiscode)) {
	  if (font->p_font_set[i].name != NULL) {
	      val = VF_GetBitmap(jiscode, font->p_font_set[i].FD,
				 w, h, bw, bo, bm_buf);
	  }
	  break;
      }
  }
  return val;
}


Private long*
GetOutline(obj, jiscode)
  FontObj  *obj;
  int      jiscode;
{
  long  *outline = NULL;
  Font  *font;
  int i;
  
  font = (Font*) obj->Locals;
  for (i = 0; i < P_FONT_MAX; i++) {
      if (is_in(font->p_font_set[i].range_set, jiscode)) {
	  if (font->p_font_set[i].name != NULL) {
	      outline = VF_GetOutline(jiscode, font->p_font_set[i].FD);
	  }
	  break;
      }
  }
  return outline;
}


Private long*
GetOutline2(obj, jiscode)
  FontObj  *obj;
  int      jiscode;
{
  long  *outline = NULL;
  Font  *font;
  int i;
  
  font = (Font*) obj->Locals;
  for (i = 0; i < P_FONT_MAX; i++) {
      if (is_in(font->p_font_set[i].range_set, jiscode)) {
	  if (font->p_font_set[i].name != NULL) {
	      outline = VF_GetOutline2(jiscode, font->p_font_set[i].FD);
	  }
	  break;
      }
  }
  return outline;
}


Private int
DrawOutline(obj, vfdata, w, h, bw, bo, bm_buf)
  FontObj  *obj;
  long     *vfdata;
  int   w;
  int   h;
  int   bw;
  int   bo;
  char  *bm_buf;
{
  int   jiscode;
  Font  *font;
  int   val = -1;
  int i;
  
  font = (Font*) obj->Locals;
  jiscode = vfdata[0];
  for (i = 0; i < P_FONT_MAX; i++) {
      if (is_in(font->p_font_set[i].range_set, jiscode)) {
	  if (font->p_font_set[i].name != NULL) {
	      val = VF_DrawOutline(vfdata, font->p_font_set[i].FD,
				   w, h, bw, bo, bm_buf);
	  }
	  break;
      }
  }
  return val;
}


Private int
FreeOutline(obj, vfdata)
  FontObj  *obj;
  long*    vfdata;
{
  Font  *font;
  int   val = -1;
  int i;

  font = (Font*) obj->Locals;
  for (i = 0; i < P_FONT_MAX; i++) {
      if (is_in(font->p_font_set[i].range_set, vfdata[0])) {
	  if (font->p_font_set[i].name != NULL) {
	      val = VF_FreeOutline(vfdata, font->p_font_set[i].FD);
	  }
	  break;
      }
  }
  return val;
}



Private int
Link(obj)
  FontObj  *obj;
{
  obj->LinkCount = obj->LinkCount + 1;
  return obj->LinkCount;
}

Private int
Unlink(obj)
  FontObj  *obj;
{
  obj->LinkCount = obj->LinkCount - 1;
  return obj->LinkCount;
}


static int
ReadCapa(font, ent)
  Font *font;
  char *ent;
{
  char *p;
  int i;
  RangeSet range_set;
  static char f_ent_str[] = "f?";
  static char r_ent_str[] = "r?";

  VFC_GetEntry(ent);

  for (i = 0; i < P_FONT_MAX; i++) {
      f_ent_str[1] = ('A' + i);
      r_ent_str[1] = ('A' + i);
      if ((p = VFC_GetString(f_ent_str)) == NULL)
	  font->p_font_set[i].name = NULL;
      else {
	  if ((font->p_font_set[i].name = malloc(strlen(p)+1)) == NULL)
	      return -1;  /* ERR: malloc err */
	  strcpy(font->p_font_set[i].name, p);
      }
      if ((p = VFC_GetString(r_ent_str)) == NULL)
	  font->p_font_set[i].range_set = NULL_RangeSet;
      else {
	  if ((range_set = parse_range_set(p)) == (RangeSet)NULL)
	      font->p_font_set[i].range_set = NULL_RangeSet;
	  else
	      font->p_font_set[i].range_set = range_set;
      }
  }
  return 0;
}


/*
 * is_in()
 *
 * val  RangeSet ϰϤˤ뤫Ĵ٤ؿ
 *
 * ϰϤˤС !NULL ֤
 *       ̵  NULL ֤
 *
 */

Private int
is_in(range_set, val)
    RangeSet range_set;
    int val;
{
    int start, end;
    while ((range_set->start >= 0) && (range_set->end >= 0)) {
	start = range_set->start;
	end = range_set->end;
	if ((start <= val) && (val <= end))
	    return !(NULL);
	range_set++;
    }
    return NULL;
}

/*
 * parse_range_set()
 *
 * ʸ󤬲ʸˡ˽äƤСRangeSet ֤
 * ǤʤС(RangeSet)NULL ֤
 *
 * () malloc() ǥƤΤǡƽФ¦ǲ뤳
 *
 * rangeset: range
 *            | rangeset COMMA range
 * range: num
 *        | num BAR num
 * num: XDIGIT+
 * XDIGIT: [0-9a-fA-F]
 * COMMA:  ','
 * BAR:    '-'
 *
 * example
 *   "1,b-9,1af-25" "ff-5,a,7"          --- OK
 *   "3.5-to" "-2" "2-" "1-2, 3-5"      --- BAD
 *     ^  ^^  ^       ^      ^ 
 */

Private RangeSet 
parse_range_set(str)
    char *str;
{
    int i;
    char *p, *p2;
    int val1, val2;

    Range tmp_set[RANGE_SET_MAX];
    RangeSet set;
    
    int range_no;
    int str_len;

    str_len = strlen(str);
    
    for (i = 0; i < str_len; i++) {
	p = (str+i);
	if (*p == ',')
	    *p = '\0';
	else if (!isxdigit(*p) && (*p != '-')) {
	    /* printf("warning: wrong char '%c' found.\n", *p); */
	    return (RangeSet)NULL;
	}
    }

    range_no = 0;
    i = 0;
    while (i < str_len) {
	if (range_no > RANGE_SET_MAX) {
	    /* printf("warning: too many range.\n"); */
	    return (RangeSet)NULL;
	}
	p = (str+i);
	if (*p == '\0') {
	    i++;
	    continue;
	}
	if (*p == '-') {
	    /* printf("warning: no start number.\n"); */
	    return (RangeSet)NULL;
	}
	val1 = strtol(p, &p2, 16);
	if (*p2 != '-') {
	    val2 = val1;
	} else {
	    if (*(p2+1) == '\0') {
		/* printf("warning: no last number.\n"); */
		return (RangeSet)NULL;
	    }
	    val2 = strtol((p2+1), &p2, 16);
	}
	if (*p2 != '\0') {
	    /* printf("warning: wrong format.\n"); */
	    return (RangeSet)NULL;
	}
	tmp_set[range_no].start = MIN(val1, val2);
	tmp_set[range_no].end = MAX(val1, val2);
	range_no++;
	i = i + strlen(p);
    }
    tmp_set[range_no].start = -1;
    tmp_set[range_no].end = -1;

    if (range_no < 1)
	return (RangeSet)NULL;
    set = (RangeSet)malloc((sizeof(Range) * (range_no+1)));
    if (!set)
	return (RangeSet)NULL;
    
    memcpy(set, tmp_set, (sizeof(Range) * (range_no+1)));
    return set;
}
