/*
 * VFenc.h
 *
 *  Programmmed by Werner Lemberg
 *  E-Mail:  a7621gac@awiuni11.bitnet
 *
 *  Edition History
 *  19 Feb 1996  First version.
 *
 */

/* This file is part of VFlib
 *
 * Copyright (C) 1995 Hirotsugu KAKUGAWA.   All rights reserved.
 *
 * VFlib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
 * to anyone for the consequences of using it or for whether it serves any
 * particular purpose or works at all, unless he says so in writing.  Refer
 * to the GNU General Public License for full details.
 *
 * Everyone is granted permission to copy, modify and redistribute
 * VFlib, but only under the conditions described in the GNU
 * General Public License.  A copy of this license is supposed to have been
 * given to you along with VFlib so you can know your rights and
 * responsibilities.  It should be in a file named COPYING.  Among other
 * things, the copyright notice and this notice must be preserved on all
 * copies. 
 */

#ifndef __VFLIB_VFENC_H__

#define __VFLIB_VFENC_H__

int     VFE_SearchEncoding();
int     VFE_SearchCharSet();
int     VFE_GetEncoding();
int     VFE_GetCharSet();

int     VFE_Init();
int     VFE_Deinit();
int     VFE_OpenFont();
int     VFE_CloseFont();
int     VFE_CloseAllFonts();
int     VFE_GetBitmap();
long*   VFE_GetOutline();
long*   VFE_GetOutline2();
int     VFE_DrawOutline();
int     VFE_FreeOutline();


/* font encodings */

#define ENC_EUC      1   /* JIS, KS, GB etc. */
#define ENC_SJIS     2   /* SJIS */
#define ENC_BIG5     3   /* Big 5 */
#define ENC_UNIC     4   /* Unicode */

/* font character sets */

#define CS_BIG5_HKU         0x401
#define CS_BIG5_ETEN        0x402
#define CS_CNS11643_1992    0x300
#define CS_CNS11643_1992_1  0x301
#define CS_CNS11643_1992_2  0x302
#define CS_CNS11643_1992_3  0x303
#define CS_CNS11643_1992_4  0x304
#define CS_CNS11643_1992_5  0x305
#define CS_CNS11643_1992_6  0x306
#define CS_CNS11643_1992_7  0x307
#define CS_GB2312_1980      0x101
#define CS_GB12345_1990     0x102
#define CS_JISX0208_1983    0x001
#define CS_JISX0212_1990    0x002
#define CS_KSC5601_1987     0x201


/* encoding string to encoding value mapping table */

struct s_encoding_table {
  char *EncString;
  int  EncValue;
};
typedef struct s_encoding_table  EncodingTable;

extern EncodingTable  EncTable[];  /* in VFEtable.c */

/* charset string to charset value mapping table */

struct s_charset_table {
  char *CharSetString;
  int  CharSetValue;
  int  DefEncValue;  /* default encoding value */
};
typedef struct s_charset_table CharSetTable;

extern CharSetTable  ChSetTable[];  /* in VFEtable.c */


/* common conversion routines for EUC fonts */

#define KUTEN_TO_GL(CJKcode)  ((CJKcode) + 0x2020)
#define KUTEN_TO_GR(CJKcode)  ((CJKcode) + 0xA0A0)
#define GL_TO_KUTEN(CJKcode)  ((CJKcode) - 0x2020)
#define GL_TO_GR(CJKcode)     ((CJKcode) + 0x8080)
#define GR_TO_KUTEN(CJKcode)  ((CJKcode) - 0xA0A0)
#define GR_TO_GL(CJKcode)     ((CJKcode) - 0x8080)


#endif  /*__VFLIB_VFENC_H__*/
