/*
 * _VF.h 
 *
 *  Programmmed by Hirotsugu Kakugawa, Hiroshima University
 *  E-Mail:  kakugawa@se.hiroshima-u.ac.jp
 *
 *  Edition History
 *  28 Dec 1993  Added VF_GetOutline().
 *  19 Jan 1994  Added font class "VF_FONT_TT". (by I. Matsuda)
 *  20 Jan 1994  Added VF_GetOutline2().
 *  15 May 1995  Added VFFM_*().
 *  18 Nov 1995  Added Null font class.
 *  21 Nov 1995  Added BDF font class.
 *  19 Feb 1996  Added GetCharSet() and GetEnc() to FontObj structure. (WL)
 *   1 Apr 1996  Added HBF font class. (WL)
 */


/* This file is part of VFlib
 *
 * Copyright (C) 1993,1994,1995 Hirotsugu KAKUGAWA.   All rights reserved.
 *
 * VFlib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
 * to anyone for the consequences of using it or for whether it serves any
 * particular purpose or works at all, unless he says so in writing.  Refer
 * to the GNU General Public License for full details.
 *
 * Everyone is granted permission to copy, modify and redistribute
 * VFlib, but only under the conditions described in the GNU
 * General Public License.  A copy of this license is supposed to have been
 * given to you along with VFlib so you can know your rights and
 * responsibilities.  It should be in a file named COPYING.  Among other
 * things, the copyright notice and this notice must be preserved on all
 * copies. 
 */

#ifndef __VFLIB__VF_H__

#define __VFLIB__VF_H__

/* Font Class ID */
#define  VF_FONT_NULL         0
#define  VF_FONT_SONY         1
#define  VF_FONT_ZEIT         2
#define  VF_FONT_JG           3
#define  VF_FONT_TT           4
#define  VF_FONT_BDF          5
#define  VF_FONT_HBF          6
#define  VF_FONT_FONTWAVE     7
#define  VF_FONT_COMPOUND    20
#define  VF_FONT_BITOP       21
#define  VF_FONT_NULLFONT    22

/* temporary use */
#define  VF_FONT_COMPOUND2   23

/*---------------------------------------------------*/

#define TRUE  (0==0)
#define FALSE (0==1)

#define  Public   /**/
#define  Const    /**/
#define  Private  static
#define  Import   extern

typedef  long       FILE_Port;  
#define  NULL_PORT  (-1)


/* Font Object Structure */
struct s_font_obj {
  /* Class ID */
  int                 ClassID;
  /* Itself */
  struct s_font_obj  *Self;
  /* The number of Objects that share this object */
  int                LinkCount;
  /* Methods */
  int                (*OpenFont)();
  int                (*CloseFont)();
  int                (*GetBitmap)();
  long*              (*GetOutline)();
  long*              (*GetOutline2)();
  int                (*DrawOutline)();
  int                (*FreeOutline)();
  int                (*Link)();
  int                (*Unlink)();
  int                (*GetCharSet)();
  int                (*GetEnc)();
  /* Class Dependent Local Data */
  long               Locals; 
};
typedef struct s_font_obj   FontObj;


/* Font Class Table */
struct s_font_class_table {
  char    *ClassName;
  int     FontClass;
  FontObj *(*ObjCreater)();
};
typedef struct s_font_class_table  FontClassTable;


/* Font Table */
struct s_font_table {
  FontObj  *Fobj;   /* Font Object */
  char     *Fentry; /* Font Entry Name */
  char     *Fname;  /* Font Name */
};
typedef struct s_font_table  FontTable;


struct s_fm_tbl {
  int     opened;
  union {
    FILE    *stream;
    int     fd;
  } uu;
  char    *path;
  int    (*open_fun)();
  int    (*close_fun)();
};
typedef struct s_fm_tbl  fm_tbl;


int  VF_LinkFont();
int  VF_UnlinkFont();

char *VF_Fontname2Entry();

int         VFFM_Init();
int         VFFM_Deinit();
FILE_Port   VFFM_Intern();
FILE_Port   VFFM_InternInt();
FILE*       VFFM_FStream();
int         VFFM_FInt();
int         VFFM_UnIntern();


#endif /* __VFLIB__VF_H__ */

