/*
 * fn2ent.c  - convert font name to entry name
 *
 *  Programmmed by Hirotsugu Kakugawa, Hiroshima University
 *  E-Mail:  kakugawa@se.hiroshima-u.ac.jp
 *
 *  Edition History
 *  31 Oct 1993
 *   4 Nov 1993  Implemented VF_Fn2Ent_TeX(), improved VF_Fn2Ent_AsItIs().
 */


/* This file is part of VFlib
 *
 * Copyright (C) 1993 Hirotsugu KAKUGAWA.   All rights reserved.
 *
 * VFlib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
 * to anyone for the consequences of using it or for whether it serves any
 * particular purpose or works at all, unless he says so in writing.  Refer
 * to the GNU General Public License for full details.
 *
 * Everyone is granted permission to copy, modify and redistribute
 * VFlib, but only under the conditions described in the GNU
 * General Public License.  A copy of this license is supposed to have been
 * given to you along with VFlib so you can know your rights and
 * responsibilities.  It should be in a file named COPYING.  Among other
 * things, the copyright notice and this notice must be preserved on all
 * copies. 
 */

#include  <stdio.h>
#include  <stdlib.h>
#include  <ctype.h>
#include  "VF.h"


/* ConvFunc
 *    The function pointed to by ConvFunc :
 *      - takes a string as an argument
 *      - returns a string
 */

static char *(*ConvFunc)() = VF_Fn2Ent_AsItIs;


char*
VF_Fontname2Entry(fontname)
  char *fontname;
{
  return (ConvFunc)(fontname);
}

int
VF_SetFn2EntFunc(f)
  char *(*f)();
{
  ConvFunc = f;
  return 0;
}


char*
VF_Fn2Ent_TeX(fontname)
  char* fontname;
{
  char        *p, *q, *val;
  static char s[64];
  static char *f = NULL;

  if (f != NULL)
    free(f);
  f = NULL;
  if (strlen(fontname) < 64){
    strcpy(s, fontname);
    val = q = s;
  } else {
    if ((f = malloc(strlen(fontname) + 1)) == NULL)
      return fontname;  /* should be NULL??? */
    val= q = f;
  }
  p = fontname;
  while (*p != '\0'){
    if (isdigit(*p)){
      *q = '\0';
      break;
    }
    *q++ = *p++;
  }
  /**printf("****FONT NAME=%s\n", val);**/
  return val;
}

char*
VF_Fn2Ent_AsItIs(fontname)
  char *fontname;
{
  char        *val;
  static char s[64];
  static char *f = NULL;

  if (f != NULL)
    free(f);
  f = NULL;
  if (strlen(fontname) < 64){
    strcpy(s, fontname);
    val = s;
  } else {
    if ((f = malloc(strlen(fontname) + 1)) == NULL)
      return fontname;  /* should be NULL??? */
    strcpy(f, fontname);
    val = f;
  }
  return val;
}

